# Number of finite elements
param Nfe > 0  integer;
# Number of collocation points in a finite element
param Ncp > 0  integer;
# The length of time horizon
param Tf ;


# Index set for finite elements
set Sfe := 1..Nfe;
# Index set for collocation points in a finite element
set Scp := 1..Ncp;
# Index set for interpolation points 
set Sip := 0..Ncp;


# The interpolation points from 1 order to 6 order 
param Rho {1..6,0..6};
# The interpolation points corresponding to Ncp order
param Tau {i in Sip} := Rho[Ncp,i];


# The values of Lagragian basis functions based on Ncp order 
# collocation points with Tau = 1
param LBF_1 {i in Sip} := 
	(prod{k in Sip diff{i}}(1 -Tau[k]))/(prod{k in Sip diff{i}}(Tau[i]-Tau[k]));

param LBF_m {j in 1..1000, i in Sip} := 
	(prod{k in Sip diff{i}}(j/1000 -Tau[k]))/(prod{k in Sip diff{i}}(Tau[i]-Tau[k]));


# var P_P{i in Sfe, j in 1..1000}=sum{k in Sip}LBF_m[j,k]*P[i,k];

# The values of first derivatives of Lagragian basis functions 
# based on Ncp order collocation points with Tau {i in Sip}
param DLBF_cp {i in Scp, j in Sip} := 
	if i==j  then sum{k in Sip diff{j}}(1 /(Tau[j]-Tau[k]))
	else (prod{k in Sip diff{i,j}}(Tau[i]-Tau[k]))/(prod{k in Sip diff{j}}(Tau[j]-Tau[k]));


# Number of differential variables
param Nz > 0  integer;

# Index set for differential variables
set Snz := 1..Nz;


# Definition of differential variables on collocation points
var Z {k in Snz, Sfe, Sip};
param Z_0 {k in Snz};
param Z_Tf {k in Snz};



var DOTZ {v in Snz,  i in Sfe, j in Scp} =
	( sum{k in Sip}(DLBF_cp[j,k]*Z[v,i,k]) ) ;

#λԤֵ
var Z_P{i in Sfe, j in 1..1000}=sum{k in Sip}LBF_m[j,k]*Z[1,i,k];

################################
######### Model Parameters #########
################################
param m := 1.36;
param Sa := 6.45e-2;
param ka := 5.25e+04;;
# param Fc := 1250;
var Fc;
# param Fs := 1750;
var Fs;
param Fv := 612;
param Vs := 2.5e-4;


#sgnźƽ
param beta:=1e-4;  
#ٶǷΪ0Ĳ|Z[2,j,k]|<delta,ΪʱǾֹģ˶
param delta := 1e-4;
#ֵƽ
param seta := 1e-4;

#ѹǿ
param P{j in Sfe,k in Scp} := 68950*sin((j-1)*(Tf/Nfe)+Tau[k]*(Tf/Nfe));
#ʵλ
param Z_m{i in Sfe, j in Scp};


var z1{j in Sfe, k in Scp} = sqrt(Z[2,j,k]^2+seta)-delta;
var z3{j in Sfe, k in Scp} = Sa*P[j,k]-ka*Z[1,j,k];
var s1{j in Sfe, k in Scp} = 0.5*(z1[j,k]/sqrt(z1[j,k]^2+4*beta^2)+1);
var f1{j in Sfe, k in Scp} = -((Sa*P[j,k]-ka*Z[1,j,k]-delta)/((Sa*P[j,k]-ka*Z[1,j,k]-delta)^2+4*beta^2)^(1/2))*0.5*sqrt((z3[j,k]+Fs-sqrt((z3[j,k]-Fs)^2+seta))^2+seta);
var f2{j in Sfe, k in Scp} = -(Fc+(Fs-Fc)*exp(-(Z[2,j,k]/Vs)^2))*(Z[2,j,k]-delta)/((Z[2,j,k]-delta)^2+4*beta^2)^0.5-Fv*Z[2,j,k];
var Ff{j in Sfe, k in Scp} = f1[j,k]*(1-s1[j,k])+f2[j,k]*s1[j,k];



# Objective function
minimize obj: sum{j in Sfe,k in Scp}(Z[1,j,k]-Z_m[j,k])^2;
# minimize obj: 1;


# Constraints on Z
s.t. ODE_Z1 { j in Sfe, k in Scp}:
				DOTZ[1,j,k] = Z[2,j,k]*(Tf/Nfe);
s.t. ODE_Z2 { j in Sfe, k in Scp}:
				DOTZ[2,j,k] = ((P[j,k]*Sa-ka*Z[1,j,k]+Ff[j,k])/m)*(Tf/Nfe);			
s.t. bound1:
				1700<=Fs<=1750;
s.t. bound2:
				1200<=Fc<=1300;
				
# Initial Condition
s.t. InitialCondition {k in Snz}:
				Z[k,1,0] = Z_0[k];
						
# Continous conditions
s.t. ContinuousCondition {k in Snz, i in Sfe diff{Nfe}}:
				Z[k,i+1,0] = sum {j in Sip} (LBF_1[j]*Z[k,i,j]);